@@ -10,12 +10,12 @@ import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Encoding as TLE
 import qualified Data.Scientific as Scientific
 import qualified Data.UUID as UUID
-import qualified PostgreSQLBinary.Decoder.Atto as Atto
 import qualified PostgreSQLBinary.Decoder.Zepto as Zepto
 import qualified PostgreSQLBinary.Array as Array
 import qualified PostgreSQLBinary.Time as Time
 import qualified PostgreSQLBinary.Integral as Integral
 import qualified PostgreSQLBinary.Numeric as Numeric
+import qualified PostgreSQLBinary.Interval as Interval
 
 
 -- |
@@ -46,24 +46,7 @@ float8 =
 {-# INLINABLE numeric #-}
 numeric :: D Scientific
 numeric =
-  evalStateT $ do
-    componentsAmount <- intOfSize 2
-    pointIndex :: Int16 <- intOfSize 2
-    signCode <- intOfSize 2
-    modify (B.drop 2)
-    components <- replicateM componentsAmount (intOfSize 2)
-    signer <-
-      if | signCode == Numeric.negSignCode -> return negate
-         | signCode == Numeric.posSignCode -> return id
-         | signCode == Numeric.nanSignCode -> lift $ Left "NAN sign"
-         | otherwise -> lift $ Left $ "Unexpected sign value: " <> (fromString . show) signCode
-    let
-      c = signer $ fromIntegral $ (Numeric.mergeComponents components :: Word64)
-      e = (fromIntegral (pointIndex + 1) - length components) * 4
-      in return $ Scientific.scientific c e
-  where
-    intOfSize n =
-      lift . int =<< state (B.splitAt n)
+  flip Zepto.run (inline Zepto.numeric)
 
 
 -- * Text
@@ -155,13 +138,8 @@ interval =
     ub <- state $ B.splitAt 8
     db <- state $ B.splitAt 4
     mb <- get
-    lift $ do
-      u <- int ub
-      d <- int db
-      m <- int mb
-      return $ picosecondsToDiffTime $
-        10 ^ 6 * fromIntegral (u + 10 ^ 6 * 60 * 60 * 24 * (d + 31 * m) :: Int64)
-
+    i <- lift $ Interval.Interval <$> int ub <*> int db <*> int mb
+    return $ Interval.toDiffTime i
 
 
 -- * Misc
